package com.bweb.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.MemberValidateUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.HighMerchant;
import com.hai.entity.HighMerchantStore;
import com.hai.model.HighMerchantStoreModel;
import com.hai.model.ResponseData;
import com.hai.model.UserInfoModel;
import com.hai.service.HighMerchantService;
import com.hai.service.HighMerchantStoreService;
import com.hai.service.SecUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/9 20:58
 */
@Controller
@RequestMapping(value = "/highMerchantStore")
@Api(value = "商户门店接口")
public class HighMerchantStoreController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private HighMerchantService highMerchantService;

    @Resource
    private SecUserService secUserService;

    @RequestMapping(value="/insertMerchantStore",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加商户门店")
    public ResponseData insertMerchantStore(@RequestBody HighMerchantStoreModel highMerchantStore, HttpServletRequest request) {
        try {
            //发布人员
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getMerchant() == null) {
                log.error("HighMerchantController -> insertMerchantStore() error!","该主角色没有权限");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MENU_TREE_HAS_NOT_ERROR, "");
            }

            if (StringUtils.isBlank(highMerchantStore.getStoreKey())
                    || StringUtils.isBlank(highMerchantStore.getStoreName())
                    || StringUtils.isBlank(highMerchantStore.getTelephone())
                    || StringUtils.isBlank(highMerchantStore.getAddress())
                    || StringUtils.isBlank(highMerchantStore.getLatitude())
                    || StringUtils.isBlank(highMerchantStore.getLongitude())
                    || StringUtils.isBlank(highMerchantStore.getSecUser().getPassword())
                    || StringUtils.isBlank(highMerchantStore.getSecUser().getTelephone())
            ) {
                log.error("HighMerchantStoreController -> insertMerchantStore() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 校验账户手机号
            if (!MemberValidateUtil.validatePhone(highMerchantStore.getSecUser().getTelephone())) {
                log.error("HighMerchantStoreController --> insertMerchantStore() error!", "请输入正确的电话号码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_IS_ERROR, "");
            }

            // 校验账号手机号是否存在
            if (secUserService.findByPhone(highMerchantStore.getSecUser().getTelephone()) != null) {
                log.error("HighMerchantStoreController --> insertMerchantStore() error!", "电话号码已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_EXISTED, "");
            }

            // 校验商户
            HighMerchant merchant = highMerchantService.getMerchantById(userInfoModel.getMerchant().getId());
            if (merchant != null) {
                log.error("HighMerchantStoreController -> insertMerchantStore() error!","未找到商户");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOF_FOUND, "");
            }

            highMerchantStore.setCreateTime(new Date());
            highMerchantStore.setUpdateTime(new Date());
            highMerchantStore.setStatus(1); // 状态：0：删除，1：正常
            highMerchantStore.setCompanyId(merchant.getCompanyId());
            highMerchantStore.setMerchantId(merchant.getId());
            highMerchantStore.setOperatorId(userInfoModel.getSecUser().getId());
            highMerchantStore.setOperatorName(userInfoModel.getSecUser().getUserName());
            highMerchantStoreService.insertMerchantStore(highMerchantStore);

            return ResponseMsgUtil.success(highMerchantStore);
        } catch (Exception e) {
            log.error("HighMerchantStoreController -> insertMerchantStore() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/updateMerchantStore",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改商户门店")
    public ResponseData updateMerchantStore(@RequestBody HighMerchantStoreModel highMerchantStore, HttpServletRequest request) {
        try {
            //发布人员
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getMerchant() == null) {
                log.error("HighMerchantController -> insertMerchantStore() error!","该主角色没有权限");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MENU_TREE_HAS_NOT_ERROR, "");
            }

            if (highMerchantStore.getId() == null
                    || StringUtils.isBlank(highMerchantStore.getStoreKey())
                    || StringUtils.isBlank(highMerchantStore.getStoreName())
                    || StringUtils.isBlank(highMerchantStore.getTelephone())
                    || StringUtils.isBlank(highMerchantStore.getAddress())
                    || StringUtils.isBlank(highMerchantStore.getLatitude())
                    || StringUtils.isBlank(highMerchantStore.getLongitude())
                    || StringUtils.isBlank(highMerchantStore.getSecUser().getTelephone())
            ) {
                log.error("HighMerchantStoreController -> insertMerchantStore() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 校验账户手机号
            if (!MemberValidateUtil.validatePhone(highMerchantStore.getSecUser().getTelephone())) {
                log.error("BsStudentController --> addStudent() error!", "请输入正确的电话号码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_IS_ERROR, "");
            }

            // 查询门店
            HighMerchantStoreModel store = highMerchantStoreService.getMerchantStoreById(highMerchantStore.getId());
            if (store == null) {
                log.error("BsStudentController --> addStudent() error!", "请输入正确的电话号码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_IS_ERROR, "");
            }

            // 校验账号手机号是否存在
            if (!store.getSecUser().getTelephone().equals(highMerchantStore.getSecUser().getTelephone())
                    && secUserService.findByPhone(highMerchantStore.getSecUser().getTelephone()) != null) {
                log.error("BsStudentController --> addStudent() error!", "电话号码已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_EXISTED, "");
            }

            // 校验商户
            HighMerchant merchant = highMerchantService.getMerchantById(highMerchantStore.getMerchantId());
            if (merchant != null) {
                log.error("HighMerchantStoreController -> editMerchantStore() error!","未找到商户");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOF_FOUND, "");
            }

            highMerchantStore.setUpdateTime(new Date());
            highMerchantStore.setOperatorId(userInfoModel.getSecUser().getId());
            highMerchantStore.setOperatorName(userInfoModel.getSecUser().getUserName());
            highMerchantStoreService.updateMerchantStore(highMerchantStore);

            return ResponseMsgUtil.success(highMerchantStore);
        } catch (Exception e) {
            log.error("HighMerchantStoreController -> editMerchantStore() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/deleteMerchantStore",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "删除商户门店")
    public ResponseData deleteMerchantStore(@RequestParam(name = "id", required = true) Long id) {
        try {
            // 商户门店
            HighMerchantStore merchantStore = highMerchantStoreService.getMerchantStoreById(id);
            if(merchantStore == null) {
                log.error("HighMerchantStoreController -> deleteMerchantStore() error!","未找到商户门店");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_STORE_NOF_FOUND, "");
            }
            highMerchantStoreService.deleteMerchantStore(id);

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> deleteMerchantStore() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getMerchantStoreById",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询商户门店")
    public ResponseData getMerchantStoreById(@RequestParam(name = "id", required = true) Long id) {
        try {

            return ResponseMsgUtil.success(highMerchantStoreService.getMerchantStoreById(id));

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> getMerchantStoreById() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getStoreListByCompany",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据公司 查询商户门店列表")
    public ResponseData getStoreListByCompany(@RequestParam(name = "merchantId", required = false) Long merchantId,
                                              @RequestParam(name = "telephone", required = false) String telephone,
                                              @RequestParam(name = "storeName", required = false) String storeName,
                                              @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                              @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                             HttpServletRequest request) {
        try {
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getBsCompany() == null) {
                log.error("HighMerchantStoreController -> getStoreListByCompany() error!","权限不足");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMPETENCE_INSUFFICIENT, "");
            }

            Map<String, Object> map = new HashMap<>();
            map.put("companyId", userInfoModel.getBsCompany().getId());
            map.put("merchantId", merchantId);
            map.put("telephone", telephone);
            map.put("storeName", storeName);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(highMerchantStoreService.getMerchantStoreList(map)));

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> getStoreListByCompany() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getStoreListByMerchant",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据商户 查询门店列表")
    public ResponseData getStoreListByMerchant(@RequestParam(name = "telephone", required = false) String telephone,
                                               @RequestParam(name = "storeName", required = false) String storeName,
                                               @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                               @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                               HttpServletRequest request) {
        try {
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getMerchant() == null) {
                log.error("HighMerchantStoreController -> getStoreListByMerchant() error!","权限不足");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMPETENCE_INSUFFICIENT, "");
            }

            Map<String, Object> map = new HashMap<>();
            map.put("merchantId", userInfoModel.getMerchant().getId());
            map.put("telephone", telephone);
            map.put("storeName", storeName);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(highMerchantStoreService.getMerchantStoreList(map)));

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> getStoreListByMerchant() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
